"""
    This module contain useful function file managing or logging
"""
import os
import re


def purge(directory, pattern):
    """
        Delete recursively all the files that match the current pattern

            :Example:
                >>> # will delete all *.foobar file in the current working directory
                >>> file_name = 'foo.foobar'
                >>> with open(file_name, 'w') as f:
                ...     f.write('foo bar')
                ...     f.close()
                ... 
                >>> os.path.exists(file_name)
                True
                >>> purge('.', '.*\.foobar')
                >>> os.path.exists(file_name)
                False


    :param directory: the directory to scan
    :param pattern: the matching pattern
    """

    for root, _, files in os.walk(directory):
        for f in os.listdir(root):
            if re.search(pattern, f):
                os.remove(os.path.join(root, f))


def create_dirs_if_not_exists(the_path):
    """
     Create all the directory tree of not exists

    :param the_path: the path to create if not exists
    """
    if not os.path.exists(os.path.abspath(the_path)):
        os.makedirs(the_path)



import glob
import sys

import serial


def get_available_usb():
    """
        Return a list of available serial USB port, works on win, macosx and linux

    :return: list of available serial USB port
    """
    if sys.platform.startswith('win'):
        ports = ['COM{}'.format(n) for n in range(1, 256)]
    elif sys.platform.startswith('darwin'):
        ports = glob.glob('/dev/tty.*')
    elif sys.platform.startswith('linux'):
        ports = glob.glob('/dev/tty[A-Za-z]*')
    else:
        msg = '{} is not a supported platform'.format(sys.platform)
        raise EnvironmentError(msg)
    available_usb = list()
    for port in ports:
        try:
            s = serial.Serial(port)
            s.close()
            available_usb.append(port)
        except (OSError, serial.SerialException):
            pass
    return available_usb


def prompt_available_usb():
    """
        Prompt in console the choice between the available usb port
        if only one is found, return it.
        If there is no available port, return None

        /!\ pyserial already  has a port listing function:
        >>> from serial.tools import list_ports
        >>> print list_ports.comports()
        [...]

    :return: str
    """
    available_usb = get_available_usb()
    if not available_usb:
        return None
    if len(available_usb) == 1:
        return available_usb[0]
    print 'Available port:'
    for i in range(len(available_usb)):
        print '\t{}. {}'.format(i + 1, available_usb[i])
    choice = 0
    correct_input = False
    while not correct_input:
        try:
            choice = int(raw_input('Choose the USB port (0 to quit): ')) - 1
            if choice == -1:
                exit(0)
            if choice in range(0, len(available_usb)):
                correct_input = True
        except ValueError:
            pass
    return available_usb[choice]



from setuptools import setup, find_packages

version = '0.3.5'
author = 'Blaise Fulpin'

setup(
    name='such_amaze_lib',
    version=version,
    packages=find_packages(),
    author=author,
    author_email='blaise.fulpin@gmail.com',
    description='Such Amaze Library, Much Wow Functions',
    long_description=open('README.rst').read(),
    include_package_data=True,
    url='https://bitbucket.org/BlaiseFulpin/such_amaze_lib',
    classifiers=[
        "Programming Language :: Python",
        "Natural Language :: French",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 2.7",
    ]
)



"""
    contain useful decorators
"""
import time


def wrapper_printer(before='Before', after='After'):
    """
        Decorator that print before and after the function execution
    :param before: printed before the execution of the function
    :param after: printed after the execution of the function

        :Example:

            >>> @wrapper_printer('hello', 'goodbye')
            ... def foo():
            ...     print 'bar'
            ... 
            >>> foo()
            hello
            bar
            goodbye
    """

    def func_wrapper(func):
        def wrapper(*args, **kwargs):
            print before
            fun_return = func(*args, **kwargs)
            print after
            return fun_return

        return wrapper

    return func_wrapper


def wrapper_timer(func):
    """
        Print on console the time a function spend executing

        :Example:

            >>> import time
            >>> @wrapper_timer
            ... def foo():
            ...     time.sleep(.42)
            ...     return 'bar'
            >>> foo()
            foo executed in ...s
            'bar'
    """

    def wrapper(*args, **kwargs):
        t = time.time()
        fun_return = func(*args, **kwargs)
        d = time.time() - t
        print '{} executed in {}s'.format(func.__name__, round(d, 2))
        return fun_return

    return wrapper


def wrapper_cache(func):
    """
        Create a cache that memorize the results each function call

        :Example:

            >>> @wrapper_cache
            ... def wait(t):
            ...     print 'waiting {}s...'.format(t)
            ...     import time
            ...     time.sleep(t)
            ...     return t
            ... 
            >>> wait(1)
            waiting 1s...
            1
            >>> wait(1)
            1

    """
    _cache = dict()

    def wrapper(*args, **kwargs):
        key = str(args) + str(kwargs)
        if not _cache.get(key):
            _cache[key] = func(*args, **kwargs)
        return _cache[key]

    return wrapper



"""
    Various XML parser en file generator
"""
from xml.etree import ElementTree
from xml.dom import minidom


class XMLToolsError(Exception):
    pass


def prettify_xml(src_filename, dst_filename):
    """
        Prettify an xml file

    :param src_filename: the source file name
    :param dst_filename: the destination file name
    """
    xml = minidom.parse(src_filename)
    pretty_xml_as_string = xml.toprettyxml()
    f = open(dst_filename, 'w')
    f.write(pretty_xml_as_string)
    f.close()


def get_xml_item(label, xml_file):
    """
        Find text text corresponding to the item label

    :param label: the item label to look for
    :param xml_file: the file to browse
    :return: str
    """
    tree = ElementTree.parse(xml_file)
    root = tree.getroot()
    for i in root.findall(label):
        return i.text


def gen_xml_file(dst_file, items):
    r"""
        Create an xml file according to the dictionary given

    :param dst_file: the xml_file name
        :Example:

        >>> items = {'port': 'COM10',
        ...         'conf_files':
        ...             {'paths': 'conf_path.xml',
        ...             'exe': 'win.exe'},
        ...         'timeout': 10}
        ...
        >>> gen_xml_file('test.xml', items)
        >>> with open('test.xml') as f:
        ...     for line in f:
        ...         line
        ...
        '<?xml version="1.0" ?>\n'
        '<data>\n'
        '\t<conf_files>\n'
        '\t\t<paths>conf_path.xml</paths>\n'
        '\t\t<exe>win.exe</exe>\n'
        '\t</conf_files>\n'
        '\t<port>COM10</port>\n'
        '\t<timeout>10</timeout>\n'
        '</data>\n'

    :return: the file abs path
        """

    try:
        root = ElementTree.Element('data')
    except Exception as e:
        raise XMLToolsError('{}: Bad argument format: {}'.format(e, items))

    def tree_build(values, parent):
        for key, value in values.items():
            if isinstance(value, dict):
                tree_build(value, ElementTree.SubElement(parent, key))
            else:
                ElementTree.SubElement(parent, key).text = str(value)

    tree_build(items, root)

    tree = ElementTree.ElementTree(root)
    tree.write(dst_file)
    prettify_xml(dst_file, dst_file)
